package com.soft.aop;

import com.alibaba.druid.util.StringUtils;
import com.soft.annotations.HasPermission;
import com.soft.service.PermissionService;
import com.sun.tools.example.debug.expr.ExpressionParser;
import org.apache.hadoop.hbase.security.AccessDeniedException;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.DefaultParameterNameDiscoverer;
import org.springframework.expression.EvaluationContext;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;


import java.lang.reflect.Method;

@Aspect
public class PermissionAspect {

    @Autowired
    private PermissionService permissionService; // 权限服务（需自行实现）

    public PermissionAspect() {
    }

    @Pointcut("@annotation(com.soft.annotations.HasPermission) && execution(public * *(..))")
    public void permissionPointCut() {
    }

    @Around("permissionPointCut()")
    public Object permission(ProceedingJoinPoint joinPoint) throws Throwable {
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        HasPermission hasPermission = method.getAnnotation(HasPermission.class);

        // 1. 解析用户ID（通过SpEL表达式）
        String userIdSpEL = hasPermission.userId();
        String userId = parseUserId(joinPoint, method, userIdSpEL);

        // 2. 校验权限
        if (!permissionService.hasPermission(
                userId,
                hasPermission.value(),
                hasPermission.calcType())) {
            throw new AccessDeniedException("用户[" + userId + "]无操作权限");
        }

        // 3. 执行目标方法
        return joinPoint.proceed();
    }

    /**
     * 解析SpEL表达式获取用户ID
     */
    private String parseUserId(ProceedingJoinPoint joinPoint, Method method, String spEL) {
        if (StringUtils.isEmpty(spEL)) {
            // 默认从安全上下文获取当前用户ID（例如Spring Security）
            Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
            return authentication.getName();
        }

        // 使用Spring的SpEL解析器
        SpelExpressionParser parser = new SpelExpressionParser();
        EvaluationContext context = new StandardEvaluationContext();

        // 将方法参数绑定到上下文
        Object[] args = joinPoint.getArgs();
        String[] paramNames = new DefaultParameterNameDiscoverer().getParameterNames(method);
        for (int i = 0; i < args.length; i++) {
            context.setVariable(paramNames[i], args[i]);
        }
        // 解析表达式并获取用户ID
        return parser.parseExpression(spEL).getValue(context, String.class);
    }
}

