package com.soft.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.NumberUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.pagehelper.PageInfo;
import com.soft.common.util.DateUtils;
import com.soft.entitys.OrderPayType;
import com.soft.entitys.sale.SaleOrderDetailBundle;
import com.soft.entitys.voice.Product;
import com.soft.entitys.voice.SaleOrder;
import com.soft.entitys.voice.SaleOrderDetail;
import com.soft.enums.ProductType;
import com.soft.enums.OrderStatus;
import com.soft.events.order.impl.ApprovePassSaleOrderEvent;
import com.soft.mapper.SaleOrderMapper;
import com.soft.service.*;
import com.soft.utils.PageResult;
import com.soft.utils.PageResultUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@Service
public class SaleOrderServiceImpl implements SaleOrderService {

    @Autowired
    private SaleOrderMapper saleOrderMapper;

    @Autowired
    private ProductService productService;

    @Autowired
    private SaleOrderDetailBundleService saleOrderDetailBundleService;

    @Autowired
    private SaleOrderDetailService saleOrderDetailService;

    @Autowired
    private OrderPayTypeService orderPayTypeService;

    @Autowired
    private ApplicationEventPublisher eventPublisher;




    @Override
    public Integer getOrderSumByStatus(Integer status) {
        return saleOrderMapper.getOrderSumByStatus(status);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void approve(String orderId, String sysUserId) {

        SaleOrder order = saleOrderMapper.selectById(orderId);
        if (order == null) {
            throw new RuntimeException("订单不存在！");
        }

        if (order.getStatus() != OrderStatus.CREATED.getCode()
                && order.getStatus() != OrderStatus.APPROVE_REFUSE.getCode()) {

            if (order.getStatus() == OrderStatus.APPROVE_PASS.getCode()) {
                throw new RuntimeException("订单已审核通过，不允许继续执行审核！");
            }
            throw new RuntimeException("订单无法审核通过！");
        }

        order.setStatus(OrderStatus.APPROVE_PASS.getCode());

        List<Integer> statusList = new ArrayList<>();
        //待审核 和审核通过
        statusList.add(OrderStatus.CREATED.getCode());
        statusList.add(OrderStatus.APPROVE_REFUSE.getCode());

        int update = saleOrderMapper.updateApproveById(sysUserId, LocalDateTime.now(), order.getId(), statusList);

        if (update != 1) {
            throw new RuntimeException("订单信息已过期，请刷新重试！");
        }
        //订单金额大于 0，校验是否存在支付方式
        if (order.getTotalAmount().compareTo(BigDecimal.ZERO) > 0) {
            List<OrderPayType> orderPayTypes = orderPayTypeService.findByOrderId(order.getId());
            if (CollectionUtil.isEmpty(orderPayTypes)) {
                //如果没有找到支付方式，抛出异常提示用户检查支付信息
                throw new RuntimeException("单据没有约定支付，请检查！");
            }
        }
        //按订单号查询所有订单详情，并按顺序处理。
        List<SaleOrderDetail> details = saleOrderDetailService.listByOrderIdAsc(order.getId());

        //遍历每个订单详情，累加总金额、普通商品数量和赠品数量
        int totalNum = 0;
        int giftNum = 0; //
        BigDecimal totalAmount = BigDecimal.ZERO; //商品的总金额

        for (SaleOrderDetail detail : details) {
            boolean isGift = detail.getIsGift();
            // 计算单个商品的总金额：含税单价 × 购买数量。
            totalAmount = NumberUtil.add(totalAmount,
                    NumberUtil.mul(detail.getTaxPrice(), detail.getOrderNum()));

            Product product = productService.findById(detail.getProductId());
            if (product.getProductType() == ProductType.NORMAL.getCode()) {
                if (isGift) {
                    giftNum += detail.getOrderNum();
                } else {
                    totalNum += detail.getOrderNum();
                }
            } else {
                //组合商品
                // 根据orderId 和detailId 查询组合商品；将组合商品存入销售单详情中，并替换掉原组合商品记录（拆细）
                List<SaleOrderDetailBundle> saleOrderDetailBundles = saleOrderDetailBundleService.listByOrderDetailId(order.getId(), detail.getId());
                if(CollectionUtil.isEmpty(saleOrderDetailBundles)){
                    return;
                }

                for (SaleOrderDetailBundle saleOrderDetailBundle : saleOrderDetailBundles) {
                    SaleOrderDetail newDetail = new SaleOrderDetail();
                    // 创建 Snowflake 实例
                    Snowflake snowflake = IdUtil.createSnowflake(1, 1);
                    newDetail.setId(String.valueOf(snowflake.nextId()));
                    newDetail.setOrderId(order.getId());
                    newDetail.setProductId(saleOrderDetailBundle.getProductId());
                    newDetail.setOrderNum(saleOrderDetailBundle.getProductOrderNum());
                    newDetail.setOriPrice(saleOrderDetailBundle.getProductOriPrice());
                    newDetail.setTaxPrice(saleOrderDetailBundle.getProductTaxPrice());
                    newDetail.setDiscountRate(detail.getDiscountRate());
                    newDetail.setIsGift(detail.getIsGift());
                    newDetail.setTaxRate(saleOrderDetailBundle.getProductTaxRate());
                    newDetail.setDescription(detail.getDescription());
                    newDetail.setOrderNo(detail.getOrderNo());
                    newDetail.setOriBundleDetailId(detail.getId());

                    saleOrderDetailService.save(newDetail);
                    saleOrderDetailService.removeById(detail.getId());

                    saleOrderDetailBundle.setProductDetailId(newDetail.getId());
                    saleOrderDetailBundleService.updateById(saleOrderDetailBundle);

                    if (isGift) {
                        giftNum += newDetail.getOrderNum();
                    } else {
                        totalNum += newDetail.getOrderNum();
                    }
                }
            }
        }
        // 这里需要重新统计明细信息，因为明细发生变动了
        SaleOrder saleOrder = new SaleOrder();
        saleOrder.setTotalNum(totalNum);
        saleOrder.setTotalGiftNum(giftNum);
        saleOrder.setTotalAmount(totalAmount);
        saleOrder.setId(order.getId());
        saleOrder.setStatus(OrderStatus.APPROVE_PASS.getCode());
        saleOrderMapper.updateById(saleOrder);

        this.sendApprovePassEvent(saleOrder);
    }

    @Override
    public PageResult<SaleOrder> pendApproveList(int pageIndex, int pageSize, Integer status) {
        int offset = (pageIndex - 1) * pageSize;
        Integer total = saleOrderMapper.getOrderSumByStatus(status);
        //根据状态查询订单列表
        List<SaleOrder> datas = saleOrderMapper.selectListByStatus(pageSize, offset, status);
        PageResult<SaleOrder> convert = PageResultUtil.convert(new PageInfo<>(datas));
        convert.setTotalCount(total);
        return convert;
    }

    @Override
    public SaleOrder selectById(String id) {
        return saleOrderMapper.selectById(id);
    }

    @Override
    public int updatePurchaseStatusbyId(SaleOrder saleOrder) {

        UpdateWrapper<SaleOrder> updateWrapper = new UpdateWrapper<>();
        updateWrapper
                .eq("id", saleOrder.getId())
                .set("purchase_status", saleOrder.getStatus());

        return saleOrderMapper.update(null, updateWrapper);
    }


    private void sendApprovePassEvent(SaleOrder order) {
        ApprovePassSaleOrderEvent event = new ApprovePassSaleOrderEvent(this);
        event.setId(order.getId());
        event.setTotalAmount(order.getTotalAmount());
        event.setApproveTime(DateUtils.toLocalDateTime(order.getApproveTime()));
        eventPublisher.publishEvent(event);  // 直接调用 Spring 方法
    }
}
