package com.soft.utils;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DateParserUtil {
    private static final DateTimeFormatter STANDARD_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd");

    // 支持的中文日期格式正则
    private static final Pattern[] DATE_PATTERNS = {
            Pattern.compile("(\\d{1,2})月(\\d{1,2})[日号]"),  // 4月25日 或 4月25号
            Pattern.compile("(\\d{4})年(\\d{1,2})月(\\d{1,2})[日号]"), // 2025年4月25日
            Pattern.compile("(\\d{1,2})\\/(\\d{1,2})"),  // 4/25
            Pattern.compile("(\\d{4})\\/(\\d{1,2})\\/(\\d{1,2})")  // 2025/4/25
    };

    /**
     * 将中文日期转换为标准格式
     * @param chineseDate 中文日期字符串
     * @return 标准格式日期字符串 (yyyy-MM-dd)
     */
    public static String parseChineseDate(String chineseDate) {
        // 统一处理输入
        String normalized = chineseDate.replaceAll("\\s+", "")
                .replace("号", "日");

        try {
            // 处理"4月28日"格式
            if (normalized.matches("\\d{1,2}月\\d{1,2}日")) {
                String[] parts = normalized.split("月|日");
                int month = Integer.parseInt(parts[0]);
                int day = Integer.parseInt(parts[1]);
                return LocalDate.now()
                        .withMonth(month)
                        .withDayOfMonth(day)
                        .format(STANDARD_FORMATTER);
            }

            // 处理"2024年4月28日"格式
            if (normalized.matches("\\d{4}年\\d{1,2}月\\d{1,2}日")) {
                String[] parts = normalized.split("年|月|日");
                int year = Integer.parseInt(parts[0]);
                int month = Integer.parseInt(parts[1]);
                int day = Integer.parseInt(parts[2]);
                return LocalDate.of(year, month, day)
                        .format(STANDARD_FORMATTER);
            }
        } catch (Exception e) {
            // 记录错误
            System.err.println("日期解析错误: " + chineseDate);
        }

        // 默认返回当天
        return LocalDate.now().format(STANDARD_FORMATTER);
    }
}
